import socket
import select

import level
import network

class Client:
	def __init__(self, server_ip):
		self.socket = socket.socket()
		self.socket.settimeout(1.0) # one second timeout
		self.socket.connect((server_ip, network.default_server_port))
		self.client_id = network.get_client_information(self.socket)
		self.ship_types = [0,0,0,0]
	
	def get_ship_type(self, player_id):
		return self.ship_types[player_id]
	
	def get_client_id(self):
		return self.client_id
	
	def send_updates(self, ship_index, key_index, is_down):
		network.send_key_information(self.socket, ship_index, key_index, is_down)
	
	def do_frame(self, level_obj, time_passed):
		#process all messages from server
		while True:
			if select.select([self.socket],[],[],0)[0] == []:
				break
			command = network.get_command_code(self.socket)
			if command == network.KEY_UPDATE_CODE:
				#key update
				ship_index, key_index, is_down = network.get_key_information(self.socket)
				level_obj.ships[ship_index].keys_pressed[key_index] = is_down
			elif command == network.SHIP_INFORMATION_CODE:
				#ship information update
				ship_code = network.get_ship_code(self.socket)
				network.get_ship_information(self.socket, level_obj.ships[ship_code])
			elif command == network.START_GAME_CODE:
				self.ship_types = network.get_ship_types(self.socket)
				level_obj.next_level = level.Level(self)
			elif command == network.WINNERS_CODE:
				winners = network.get_winners(self.socket)
				level_obj.winners = winners
			elif command == '':
				#connection closed
				raise Exception("Connection to server was lost")
			else:
				raise Exception("Unknown command code: %s" % command)

	def handle_level_name(self):			
		return network.get_string(self.socket)
	
	def handle_ship_dead(self, dead_ship, level):
		pass
	
	def check_start_game(self):
		if select.select([self.socket],[],[],0)[0] == []:
			return False
		command = network.get_command_code(self.socket)
		if command == network.START_GAME_CODE:
			self.ship_types = network.get_ship_types(self.socket)
			return True
		else:
			raise Exception("Unrecognized command code from server")
	
	def __del__(self):
		self.socket.close()
