import socket
import select
import random

import network
import level
import sprite
import ship

level_options = ["level1","level2","level3"]

class Server:
	def __init__(self):
		self.listening_socket = socket.socket()
		self.listening_socket.bind(("0.0.0.0", network.default_server_port))
		self.listening_socket.listen(3) #maximum 3 connections are cached
		self.listening_socket.setblocking(False)
		
		self.client_sockets = {} #socket: ship_id
		self.free_ids = [2,1,3]
		
		self.time_since_last_update = 0
		self.update_first_pair = True #alternating
		self.ship_types = [0,0,0,0]
	
	def get_client_id(self):
		return 0
	
	def get_ship_type(self, player_id):
		return self.ship_types[player_id]
	
	def check_new_client(self,level_obj):
		try:
			new_sock,address = self.listening_socket.accept()
			if len(self.free_ids) == 0:
				new_sock.close()
				return
			client_id = self.free_ids[0]
			network.send_client_information(new_sock, client_id)
			self.client_sockets[new_sock] = client_id
			level_obj.ship_labels[client_id] = sprite.SpriteImage.getTextSpriteImage(
				"Ship %d: Client" % (client_id+1), font_size = 24)
			self.free_ids.remove(client_id)
		except socket.error, e:
			pass
	
	def stop_listening(self):
		try:
			self.listening_socket.close()
		except:
			pass
	
	def send_updates(self, ship_index, key_index, is_down):
		dead_clients = []
		for sock in self.client_sockets.keys():
			try:
				network.send_key_information(sock, ship_index, key_index, is_down)
			except:
				self._remove_client(sock)
	
	def send_start_game(self):
		for sock in self.client_sockets.keys():
			try:
				network.send_start_game(sock,self.ship_types)
			except:
				self._remove_client(sock)
	
	def _remove_client(self, dead_client):
		try:
			dead_client.close()
		except:
			pass
		self.free_ids.append(self.client_sockets[dead_client])
		del self.client_sockets[dead_client]
	
	def handle_level_name(self):
		level_name = random.choice(level_options)
		for sock in self.client_sockets.keys():
			try:
				network.send_string(sock, level_name)
			except:
				self._remove_client(sock)
		return level_name
	
	def _send_ship_info(self, level, ship_index):
		ship_to_update = level.ships[ship_index]
		for client_socket in self.client_sockets.keys():
			try:
				network.send_ship_information(client_socket, ship_index, ship_to_update)
			except:
				self._remove_client(client_socket)

	def _send_winners(self, winners):
		for client_socket in self.client_sockets.keys():
			try:
				network.send_winners(client_socket, winners)
			except:
				self._remove_client(client_socket)
	
	def handle_ship_dead(self, dead_ship, level):
		if dead_ship.is_dead == True:
			return
		if dead_ship.ship_type == len(ship.ship_types)-1:
			return
		dead_ship.is_dead = True
		level.level_ended = True
		if level.winners == None:
			level.winners = [(1-dead_ship.index/2)*2,(1-dead_ship.index/2)*2+1]
			self._send_winners(level.winners)
		self._send_ship_info(level, dead_ship.index)
	
	def next_level(self, level_obj):
		self.send_start_game()
		level_obj.next_level = level.Level(self)
	
	def do_frame(self, level, time_passed):
		#first, check for updates from the clients
		while (True):
			ready_to_recv = []
			if self.client_sockets.keys() != []:
				ready_to_recv = select.select(self.client_sockets.keys(),[],[],0)[0]
			if ready_to_recv == []:
				break
			#pass on each socket once(one command) every loop, until all processing is done
			for client_socket in ready_to_recv:
				try:
					command = network.get_command_code(client_socket)
					if command == network.KEY_UPDATE_CODE:
						ship_index, key_index, is_down = network.get_key_information(client_socket)
						level.ships[ship_index].keys_pressed[key_index] = is_down
						for other_client in filter(lambda s: s!= client_socket, self.client_sockets.keys()):
							network.send_key_information(other_client, ship_index, key_index, is_down)
					elif command == 'Q':
						pass #end game?
					else:
						raise Exception("Unrecognized protocol")
				except Exception, e:
					#when a client does a problem, kick him out
					print "Error handling a client message: %s\n" % e
					self._remove_client(client_socket)
		
		#send regular updates to clients about ship position and other information
		self.time_since_last_update += time_passed
		if self.time_since_last_update > 0.13:
			self.time_since_last_update = 0
			
			if self.update_first_pair:
				self._send_ship_info(level, 0)
				self._send_ship_info(level, 1)
			else:
				self._send_ship_info(level, 2)
				self._send_ship_info(level, 3)
			
			self.update_first_pair = not self.update_first_pair
	
	def __del__(self):
		try:
			for sock in self.client_sockets.keys():
				try:
					sock.close()
				except:
					pass
		except:
			pass
		try:
			self.listening_socket.close()
		except:
			pass