import math

from OpenGL import GL

import sprite
import data


rotation_speed = 2.57 #rads/sec
acceleration_speed = 6.5 #units/sec^2
slowdown_rate = 0.3 #exponent factor

ship_types = [
	"Battlestar Galactica",
	"Defiant",
	"Discovery",
	"Eagle",
	"Raptor",
	"Republic Attack Cruiser",
	"Republic Gunship",
	"Viper Mark II",
	"Viper Mark VII",
	"Moon"
	]

def load_ship_image(ship_type):
	filename = ship_types[ship_type].replace(" ","_")
	return sprite.SpriteImage.getSpriteImage(data.filepath("ships","%s.png" % filename))

class Ship:
	def __init__(self, ship_type, position = [0,0], direction = 0, velocity = [0,0]):
		self.ship_type = ship_type
		self.ship_sprite = load_ship_image(ship_type)
		#updated every once in a while
		self.last_position = position # vector
		self.position = position #vector
		self.direction = direction #radians
		self.velocity = velocity #vector
		self.is_dead = False
		
		#updated from/to server upon change
		self.keys_pressed = [False, False, False, False] #ASDW or [left,down,right,up]
	
	def draw(self):
		GL.glPushMatrix()
		GL.glTranslatef(self.position[0],self.position[1],0)
		
		GL.glRotatef((180/math.pi)*self.direction-90, 0,0,1)
		
		if self.is_dead:
			sprite.set_color([0.5,0.5,0.5,1])
		else:
			sprite.set_color([1,1,1,1])
		GL.glTranslatef(-0.5,-0.5,0)
		
		GL.glScalef(1.0/self.ship_sprite.size[0],1.0/self.ship_sprite.size[1],1)
		self.ship_sprite.draw()
		
		GL.glPopMatrix()
	
	def do_frame(self, time_passed):
		self.last_position = self.position[:]
		self.position[0] += self.velocity[0] * time_passed
		self.position[1] += self.velocity[1] * time_passed
		
		if not self.is_dead:
			if self.keys_pressed[0]:
				self.direction -= rotation_speed * time_passed
			if self.keys_pressed[2]:
				self.direction += rotation_speed * time_passed
			
			to_accel=0
			if self.keys_pressed[1]:
				to_accel -= 1
			if self.keys_pressed[3]:
				to_accel += 1
			if 0 != to_accel:
				to_accel *= time_passed * acceleration_speed
				self.velocity[0] += math.cos(self.direction) * to_accel
				self.velocity[1] += math.sin(self.direction) * to_accel
		
		slow_down = math.exp(-time_passed * slowdown_rate)
		self.velocity[0] *= slow_down
		self.velocity[1] *= slow_down
